@@ -161,8 +161,12 @@ class AbstractMatcher {
     friend class Iterator<typename reflex::TypeOp<T>::ConstType>;
     friend class Iterator<typename reflex::TypeOp<T>::NonConstType>;
    public:
+    /// Non-const AbstractMatcher type
+    typedef typename reflex::TypeOp<T>::NonConstType NonConstT;
+    /// Const AbstractMatcher type
+    typedef typename reflex::TypeOp<T>::ConstType ConstT;
     /// Iterator iterator_category trait.
-    typedef std::output_iterator_tag iterator_category;
+    typedef std::input_iterator_tag iterator_category;
     /// Iterator value_type trait.
     typedef T value_type;
     /// Iterator difference_type trait.
@@ -178,7 +182,7 @@ class AbstractMatcher {
         method_()
     { }
     /// Copy constructor.
-    Iterator(const Iterator<typename reflex::TypeOp<T>::NonConstType>& it)
+    Iterator(const Iterator<NonConstT>& it)
       :
         matcher_(it.matcher_),
         method_(it.method_)
@@ -196,13 +200,13 @@ class AbstractMatcher {
       return matcher_;
     }
     /// AbstractMatcher::Iterator equality.
-    bool operator==(const Iterator<typename reflex::TypeOp<T>::ConstType>& rhs) const
+    bool operator==(const Iterator<ConstT>& rhs) const
       /// @returns true if iterator equals RHS
     {
       return matcher_ == rhs.matcher_;
     }
     /// AbstractMatcher::Iterator inequality.
-    bool operator!=(const Iterator<typename reflex::TypeOp<T>::ConstType>& rhs) const
+    bool operator!=(const Iterator<ConstT>& rhs) const
       /// @returns true if iterator does not equal RHS
     {
       return matcher_ != rhs.matcher_;
@@ -225,8 +229,8 @@ class AbstractMatcher {
     }
     /// Construct an AbstractMatcher::Iterator to scan, search, or split an input character sequence.
     Iterator(
-        AbstractMatcher *matcher, ///< iterate over pattern matches with this matcher
-        Method           method)  ///< match using method Const::SCAN, Const::FIND, or Const::SPLIT
+        NonConstT *matcher, ///< iterate over pattern matches with this matcher
+        Method     method)  ///< match using method Const::SCAN, Const::FIND, or Const::SPLIT
       :
         matcher_(matcher),
         method_(method)
@@ -235,8 +239,8 @@ class AbstractMatcher {
         matcher_ = NULL;
     }
    private:
-    AbstractMatcher *matcher_; ///< the matcher used by this iterator
-    Method           method_;  ///< the method for pattern matching by this iterator's matcher
+    NonConstT *matcher_; ///< the matcher used by this iterator
+    Method     method_;  ///< the method for pattern matching by this iterator's matcher
   };
  public:
   typedef AbstractMatcher::Iterator<AbstractMatcher>       iterator;       ///< std::input_iterator for scanning, searching, and splitting input character sequences
